import { State } from "."
import { Config, defaultNotifyInterval } from "./config"
import { notify } from "./notify"
import { formatTS } from "./utils"

type EndpointState = State extends Map<any, infer L> ? L : never

export const handleDown = (endpointState: EndpointState, curTime: number, endpoint: Config["pollEndpoints"][number]) => {
    const prevDown = endpointState.isDown
    endpointState.isDown = true
    if (!prevDown) endpointState.downStart = curTime

    if (curTime - endpointState.lastDownAlert < (endpoint.notifyInterval ?? defaultNotifyInterval) * 1000) return
    const message = prevDown ? 
        `[${formatTS(curTime)}] ${endpoint.name} is still down and initially went down at ${formatTS(endpointState.downStart)}` :
        `[${formatTS(curTime)}] ${endpoint.name} went down`

    notify(message, endpoint)
    endpointState.lastDownAlert = curTime
}

export const handleUp = (endpointState: EndpointState, curTime: number, endpoint: Config["pollEndpoints"][number]) => {
    if (endpointState.isDown) {
        endpointState.isDown = false
        const message = `[${formatTS(curTime)}] ${endpoint.name} is back up`
        notify(message, endpoint)
    }
}